@@ -37,9 +37,13 @@ from .core.config import DataFlowConfig
 from .core.engine import DataFlow
 from .core.model_registry import ModelRegistry
 from .core.models import DataFlowModel
+from .utils.suppress_warnings import suppress_core_sdk_warnings
+
+# Suppress verbose Core SDK warnings on import
+suppress_core_sdk_warnings()
 
 # Legacy compatibility - maintain the original imports
-__version__ = "0.8.1"
+__version__ = "0.9.0"
 
 __all__ = [
     "DataFlow",
