@@ -8,11 +8,70 @@ Automatically detects environment and configures optimal settings.
 import multiprocessing
 import os
 from dataclasses import asdict, dataclass, field
+from enum import Enum
 from pathlib import Path
 from typing import Any, Dict, List, Optional
 
 from .models import Environment
 
+# ============================================================================
+# ErrorEnhancer Performance Configuration
+# ============================================================================
+
+
+class PerformanceMode(Enum):
+    """Performance modes for ErrorEnhancer.
+
+    - FULL: Complete pattern matching, all context, all solutions (~5ms overhead)
+    - MINIMAL: Basic error code, essential context, top solution only (~1ms overhead)
+    - DISABLED: Passthrough, no enhancement overhead (~0.1ms overhead)
+    """
+
+    FULL = "FULL"
+    MINIMAL = "MINIMAL"
+    DISABLED = "DISABLED"
+
+
+@dataclass
+class ErrorEnhancerConfig:
+    """Configuration for ErrorEnhancer performance optimization.
+
+    Attributes:
+        mode: Performance mode (FULL/MINIMAL/DISABLED)
+        cache_size: Maximum number of cached pattern compilations (default: 100)
+        cache_ttl: Cache entry TTL in seconds (None = no expiration)
+    """
+
+    mode: PerformanceMode = PerformanceMode.FULL
+    cache_size: int = 100
+    cache_ttl: Optional[int] = None
+
+    @classmethod
+    def from_env(cls) -> "ErrorEnhancerConfig":
+        """Create configuration from environment variables.
+
+        Environment variables:
+            DATAFLOW_ERROR_ENHANCER_MODE: Performance mode (FULL/MINIMAL/DISABLED)
+            DATAFLOW_ERROR_ENHANCER_CACHE_SIZE: Cache size (default: 100)
+            DATAFLOW_ERROR_ENHANCER_CACHE_TTL: Cache TTL in seconds (default: None)
+
+        Returns:
+            ErrorEnhancerConfig instance
+        """
+        mode_str = os.getenv("DATAFLOW_ERROR_ENHANCER_MODE", "FULL").upper()
+        mode = (
+            PerformanceMode[mode_str]
+            if mode_str in PerformanceMode.__members__
+            else PerformanceMode.FULL
+        )
+
+        cache_size = int(os.getenv("DATAFLOW_ERROR_ENHANCER_CACHE_SIZE", "100"))
+
+        cache_ttl_str = os.getenv("DATAFLOW_ERROR_ENHANCER_CACHE_TTL")
+        cache_ttl = int(cache_ttl_str) if cache_ttl_str else None
+
+        return cls(mode=mode, cache_size=cache_size, cache_ttl=cache_ttl)
+
 
 @dataclass
 class DatabaseConfig:
