@@ -242,7 +242,10 @@ class BatchedMigrationExecutor:
             if (
                 op1.operation_type == MigrationType.ADD_COLUMN
                 and op2.operation_type
-                in [MigrationType.ADD_INDEX, MigrationType.ADD_CONSTRAINT]
+                in [
+                    MigrationType.ADD_INDEX,
+                    MigrationType.ADD_CONSTRAINT,
+                ]
             ):
                 # Check if operations might affect the same column
                 return True
