@@ -492,8 +492,8 @@ class ConcurrentMigrationQueue:
 
             # If not in queue, might be processing or completed
             return QueueStatus(
-                queue_id=queue_id, status="PROCESSING"  # Simplified status
-            )
+                queue_id=queue_id, status="PROCESSING"
+            )  # Simplified status
 
     def cancel_queued_migration(self, queue_id: str) -> bool:
         """
