@@ -646,14 +646,59 @@ class MigrationHistoryManager:
                 logger.debug(f"Migration recording successful via {attempt_name}")
                 return
             except Exception as e:
-                logger.warning(f"Migration recording failed via {attempt_name}: {e}")
+                # Enhanced error messages with context (Bug #3 fix)
+                error_context = self._get_error_context(e)
+                logger.warning(
+                    f"Migration recording failed via {attempt_name}: {e}\\n{error_context}"
+                )
                 continue
 
         # If all attempts fail, log but don't crash the migration
         logger.error(
-            "All migration recording attempts failed - migration will proceed without history recording"
+            "All migration recording attempts failed - migration will proceed without history recording\\n"
+            "IMPACT: Migration completed successfully but history not recorded\\n"
+            "CAUSE: Unable to write to migration history table\\n"
+            "FIX: Check database permissions and ensure migration tables exist"
         )
 
+    def _get_error_context(self, error: Exception) -> str:
+        """
+        Get helpful context for migration recording errors.
+
+        Args:
+            error: The exception that occurred
+
+        Returns:
+            Contextual help message
+        """
+        error_str = str(error).lower()
+
+        # Detect specific error patterns and provide context
+        if "cannot be called from a running event loop" in error_str:
+            return (
+                "CONTEXT: DataFlow called from async context (FastAPI/async app)\\n"
+                "NOTE: This error is expected and handled - trying fallback method\\n"
+                "INFO: Upgrade to DataFlow v0.7.3+ for improved async support"
+            )
+        elif "timeout" in error_str:
+            return (
+                "CONTEXT: Migration recording took too long\\n"
+                "POSSIBLE CAUSES: Database connection slow, large migration data\\n"
+                "SUGGESTION: Check database connectivity and performance"
+            )
+        elif "permission" in error_str or "access denied" in error_str:
+            return (
+                "CONTEXT: Database permission issue\\n"
+                "FIX: Ensure database user has INSERT permission on migration tables"
+            )
+        elif "no such table" in error_str or "relation" in error_str:
+            return (
+                "CONTEXT: Migration history table does not exist\\n"
+                "FIX: Ensure migration tracking table is created (auto_migrate should handle this)"
+            )
+        else:
+            return "SUGGESTION: Check database connectivity and migration table schema"
+
     def _record_migration_fallback(self, migration: MigrationRecord) -> None:
         """
         Fallback migration recording that always succeeds (possibly as no-op).
@@ -738,10 +783,17 @@ class MigrationHistoryManager:
 
     def _record_migration_with_thread_pool(self, migration: MigrationRecord) -> None:
         """
-        Record migration using thread pool to isolate async execution.
+        Record migration with automatic async/sync context detection.
+
+        This method detects whether it's running in an async context (like FastAPI)
+        and adapts accordingly to avoid "asyncio.run() cannot be called from a
+        running event loop" errors.
 
         Args:
             migration: Migration record to store
+
+        Raises:
+            Exception: If migration recording fails in all attempted methods
         """
         import asyncio
         import concurrent.futures
@@ -749,14 +801,45 @@ class MigrationHistoryManager:
         async def async_execute():
             return await self._record_migration_async(migration)
 
-        # Use thread pool to isolate the async execution
-        with concurrent.futures.ThreadPoolExecutor() as executor:
-            # Submit the async function to be run in a new event loop
-            def run_async():
-                return asyncio.run(async_execute())
+        try:
+            # Try to detect running event loop
+            loop = asyncio.get_running_loop()
 
-            future = executor.submit(run_async)
-            future.result()  # Wait for completion and re-raise any exceptions
+            # We're in an async context (FastAPI, Jupyter, etc.)
+            # Use run_coroutine_threadsafe to schedule in existing loop
+            logger.debug(
+                "Async context detected (FastAPI/async app) - using run_coroutine_threadsafe"
+            )
+
+            future = asyncio.run_coroutine_threadsafe(async_execute(), loop)
+            result = future.result(timeout=30)  # 30 second timeout
+            logger.debug("Migration recording successful in async context")
+            return result
+
+        except RuntimeError as e:
+            # No running event loop - we're in sync context
+            if "no running event loop" in str(e).lower():
+                logger.debug("Sync context detected (CLI/script) - using thread pool")
+
+                # Safe to use asyncio.run() in new thread
+                with concurrent.futures.ThreadPoolExecutor() as executor:
+
+                    def run_async():
+                        return asyncio.run(async_execute())
+
+                    future = executor.submit(run_async)
+                    result = future.result(timeout=30)  # 30 second timeout
+                    logger.debug("Migration recording successful in sync context")
+                    return result
+            else:
+                # Some other RuntimeError - re-raise
+                logger.error(f"Unexpected RuntimeError during migration recording: {e}")
+                raise
+
+        except concurrent.futures.TimeoutError:
+            error_msg = "Migration recording timed out after 30 seconds"
+            logger.error(error_msg)
+            raise Exception(error_msg)
 
     def _record_migration_sync(self, migration: MigrationRecord) -> None:
         """
