@@ -2,9 +2,7 @@
 
 from .aggregate_operations import AggregateNode
 from .mongodb_nodes import AggregateNode as MongoAggregateNode
-from .mongodb_nodes import (
-    BulkDocumentInsertNode,
-)
+from .mongodb_nodes import BulkDocumentInsertNode
 from .mongodb_nodes import CreateIndexNode as MongoCreateIndexNode
 from .mongodb_nodes import (
     DocumentCountNode,
