@@ -170,7 +170,11 @@ class BulkDeleteNode(SmartNodeConnectionMixin, AsyncNode):
                 }
 
             # Safety checks
-            if self.safe_mode and not filter_conditions and not ids:
+            # FIXED: Use key existence check instead of truthiness check
+            # Bug: `not filter_conditions` evaluates to True for empty dict {}
+            # Fix: `"filter" not in validated_inputs` checks if filter parameter was provided
+            # This matches the fix at line 153 and prevents rejecting empty filter {}
+            if self.safe_mode and "filter" not in validated_inputs and not ids:
                 raise NodeValidationError(
                     "Safe mode requires filter conditions to prevent accidental full table deletion"
                 )
