@@ -108,6 +108,13 @@ class BulkUpsertNode(SmartNodeConnectionMixin, AsyncNode):
                 default=None,
                 description="How to handle conflicts: update, ignore (overrides config)",
             ),
+            "conflict_on": NodeParameter(
+                name="conflict_on",
+                type=list,
+                required=False,
+                default=None,
+                description="Fields to detect conflicts on (overrides config conflict_columns). Example: ['email'] or ['order_id', 'product_id']",
+            ),
             "workflow_context": NodeParameter(
                 name="workflow_context",
                 type=dict,
@@ -140,6 +147,8 @@ class BulkUpsertNode(SmartNodeConnectionMixin, AsyncNode):
             merge_strategy = (
                 validated_inputs.get("merge_strategy") or self.merge_strategy
             )
+            # Use conflict_on from runtime parameter, otherwise fall back to config conflict_columns
+            conflict_on = validated_inputs.get("conflict_on") or self.conflict_columns
 
             # Validate input data
             if not data:
@@ -160,6 +169,7 @@ class BulkUpsertNode(SmartNodeConnectionMixin, AsyncNode):
                 kwargs_copy.pop("tenant_id", None)
                 kwargs_copy.pop("return_records", None)
                 kwargs_copy.pop("merge_strategy", None)
+                kwargs_copy.pop("conflict_on", None)
                 (
                     rows_affected,
                     inserted_count,
@@ -167,7 +177,12 @@ class BulkUpsertNode(SmartNodeConnectionMixin, AsyncNode):
                     upserted_records,
                     duplicates_removed,
                 ) = await self._execute_real_bulk_upsert(
-                    data, tenant_id, return_records, merge_strategy, **kwargs_copy
+                    data,
+                    tenant_id,
+                    return_records,
+                    merge_strategy,
+                    conflict_on,
+                    **kwargs_copy,
                 )
             else:
                 # Dry run or fallback mock
@@ -219,7 +234,7 @@ class BulkUpsertNode(SmartNodeConnectionMixin, AsyncNode):
                 "metadata": {
                     "table_name": self.table_name,
                     "merge_strategy": merge_strategy,
-                    "conflict_columns": self.conflict_columns,
+                    "conflict_columns": conflict_on,  # Use resolved conflict_on
                     "batch_size": self.batch_size,
                     "auto_timestamps": self.auto_timestamps,
                     "version_control": self.version_control,
@@ -271,6 +286,7 @@ class BulkUpsertNode(SmartNodeConnectionMixin, AsyncNode):
         tenant_id: Optional[str],
         return_records: bool,
         merge_strategy: str,
+        conflict_on: List[str],
         **kwargs,
     ) -> tuple[int, int, int, List[Dict[str, Any]], int]:
         """Execute real bulk upsert using database connection."""
@@ -283,7 +299,9 @@ class BulkUpsertNode(SmartNodeConnectionMixin, AsyncNode):
 
             # Remove duplicates within the batch to avoid PostgreSQL conflict errors
             original_count = len(processed_data)
-            deduplicated_data = self._deduplicate_batch_data(processed_data)
+            deduplicated_data = self._deduplicate_batch_data(
+                processed_data, conflict_on
+            )
             duplicates_removed = original_count - len(deduplicated_data)
 
             # Get column names from first record
@@ -301,7 +319,12 @@ class BulkUpsertNode(SmartNodeConnectionMixin, AsyncNode):
 
                 # Build UPSERT query
                 query = self._build_upsert_query(
-                    batch, columns, column_names, return_records, merge_strategy
+                    batch,
+                    columns,
+                    column_names,
+                    return_records,
+                    merge_strategy,
+                    conflict_on,
                 )
 
                 # Execute batch using connection pool if available, otherwise fallback
@@ -364,14 +387,14 @@ class BulkUpsertNode(SmartNodeConnectionMixin, AsyncNode):
         return processed_data
 
     def _deduplicate_batch_data(
-        self, data: List[Dict[str, Any]]
+        self, data: List[Dict[str, Any]], conflict_on: List[str]
     ) -> List[Dict[str, Any]]:
-        """Remove duplicates within batch based on conflict columns to avoid PostgreSQL errors."""
+        """Remove duplicates within batch based on conflict_on columns to avoid PostgreSQL errors."""
         if self.handle_duplicates == "last":
             # Keep the last occurrence of each unique key
             seen = {}
             for i, record in enumerate(data):
-                conflict_key = tuple(record.get(col) for col in self.conflict_columns)
+                conflict_key = tuple(record.get(col) for col in conflict_on)
                 seen[conflict_key] = i  # Keep track of latest index
 
             # Extract records using the latest indices
@@ -382,7 +405,7 @@ class BulkUpsertNode(SmartNodeConnectionMixin, AsyncNode):
             deduplicated = []
 
             for record in data:
-                conflict_key = tuple(record.get(col) for col in self.conflict_columns)
+                conflict_key = tuple(record.get(col) for col in conflict_on)
 
                 if conflict_key not in seen:
                     seen.add(conflict_key)
@@ -397,6 +420,7 @@ class BulkUpsertNode(SmartNodeConnectionMixin, AsyncNode):
         column_names: str,
         return_records: bool,
         merge_strategy: str,
+        conflict_on: List[str],
     ) -> str:
         """Build UPSERT query with proper conflict resolution."""
         # Build value rows with proper escaping
@@ -423,7 +447,7 @@ class BulkUpsertNode(SmartNodeConnectionMixin, AsyncNode):
 
         # Add conflict resolution
         if self.database_type == "postgresql":
-            conflict_columns_str = ", ".join(self.conflict_columns)
+            conflict_columns_str = ", ".join(conflict_on)
 
             if merge_strategy == "ignore":
                 query = f"{base_query} ON CONFLICT ({conflict_columns_str}) DO NOTHING"
@@ -431,7 +455,7 @@ class BulkUpsertNode(SmartNodeConnectionMixin, AsyncNode):
                 # Build update clauses, excluding conflict columns and immutable fields
                 update_clauses = []
                 for col in columns:
-                    if col not in self.conflict_columns and col not in [
+                    if col not in conflict_on and col not in [
                         "id",
                         "created_at",
                     ]:
