@@ -9,6 +9,8 @@ This test suite ensures that the following bugs remain fixed:
    - BulkCreateNode: Auto-convert datetime fields for all records
    - BulkUpdateNode: Auto-convert datetime fields in update dict
    - BulkUpsertNode: Auto-convert datetime fields for all records
+3. BUG #514: Optional[T] Type Stripping - Optional types stripped to T during node generation
+4. BUG #515: Premature JSON Serialization - Dict/list parameters prematurely serialized to JSON strings
 """
 
 import asyncio
@@ -22,11 +24,11 @@ import pytest
 # Add DataFlow to path
 sys.path.insert(0, str(Path(__file__).parent.parent / "src"))
 
-from dataflow import DataFlow
-
 from kailash.runtime import AsyncLocalRuntime
 from kailash.workflow.builder import WorkflowBuilder
 
+from dataflow import DataFlow
+
 
 class TestBugFix1_BulkOperationsStringLiteral:
     """Regression tests for BUG #1: Bulk Operations String Literal Bug"""
@@ -58,12 +60,12 @@ class TestBugFix1_BulkOperationsStringLiteral:
 
         # Execute
         runtime = AsyncLocalRuntime()
-        results = await runtime.execute_workflow_async(workflow.build(), inputs={})
+        results, run_id = await runtime.execute_workflow_async(
+            workflow.build(), inputs={}
+        )
 
         # Verify - should succeed without errors
-        # AsyncLocalRuntime returns: {'results': {...}, 'errors': {}, ...}
-        assert "results" in results
-        assert "bulk_create" in results["results"]
+        assert "bulk_create" in results
         # Note: exact result structure may vary, but it should not raise an error
 
     @pytest.mark.asyncio
@@ -92,11 +94,12 @@ class TestBugFix1_BulkOperationsStringLiteral:
 
         # Execute
         runtime = AsyncLocalRuntime()
-        results = await runtime.execute_workflow_async(workflow.build(), inputs={})
+        results, run_id = await runtime.execute_workflow_async(
+            workflow.build(), inputs={}
+        )
 
         # Should succeed without string literal errors
-        assert "results" in results
-        assert "bulk_update" in results["results"]
+        assert "bulk_update" in results
 
     @pytest.mark.asyncio
     async def test_bulk_delete_minimal_params(self, tmp_path):
@@ -119,11 +122,12 @@ class TestBugFix1_BulkOperationsStringLiteral:
 
         # Execute
         runtime = AsyncLocalRuntime()
-        results = await runtime.execute_workflow_async(workflow.build(), inputs={})
+        results, run_id = await runtime.execute_workflow_async(
+            workflow.build(), inputs={}
+        )
 
         # Should succeed
-        assert "results" in results
-        assert "bulk_delete" in results["results"]
+        assert "bulk_delete" in results
 
     @pytest.mark.asyncio
     async def test_bulk_upsert_minimal_params(self, tmp_path):
@@ -146,11 +150,12 @@ class TestBugFix1_BulkOperationsStringLiteral:
 
         # Execute
         runtime = AsyncLocalRuntime()
-        results = await runtime.execute_workflow_async(workflow.build(), inputs={})
+        results, run_id = await runtime.execute_workflow_async(
+            workflow.build(), inputs={}
+        )
 
         # Should succeed
-        assert "results" in results
-        assert "bulk_upsert" in results["results"]
+        assert "bulk_upsert" in results
 
 
 class TestBugFix2_UpdateNodeDatetimeConversion:
@@ -174,7 +179,7 @@ class TestBugFix2_UpdateNodeDatetimeConversion:
             "EventUpdateNode",
             "update_event",
             {
-                "filter": {"name": "Conference"},
+                "filter": {"id": "event-1"},  # UpdateNode requires id or record_id
                 "fields": {
                     "event_date": "2024-07-20T16:00:00"
                 },  # ISO string, not datetime object
@@ -183,11 +188,12 @@ class TestBugFix2_UpdateNodeDatetimeConversion:
 
         # Execute - should auto-convert string to datetime without errors
         runtime = AsyncLocalRuntime()
-        results = await runtime.execute_workflow_async(workflow.build(), inputs={})
+        results, run_id = await runtime.execute_workflow_async(
+            workflow.build(), inputs={}
+        )
 
         # Verify update succeeded (no JSON serialization errors)
-        assert "results" in results
-        assert "update_event" in results["results"]
+        assert "update_event" in results
 
     @pytest.mark.asyncio
     async def test_update_datetime_with_timezone(self, tmp_path):
@@ -206,15 +212,16 @@ class TestBugFix2_UpdateNodeDatetimeConversion:
             "MeetingUpdateNode",
             "update_meeting",
             {
-                "filter": {"title": "Team Sync"},
+                "filter": {"id": "meeting-1"},  # UpdateNode requires id or record_id
                 "fields": {"scheduled_at": "2024-08-01T10:00:00Z"},  # With Z timezone
             },
         )
 
         runtime = AsyncLocalRuntime()
-        results = await runtime.execute_workflow_async(workflow.build(), inputs={})
-        assert "results" in results
-        assert "update_meeting" in results["results"]
+        results, run_id = await runtime.execute_workflow_async(
+            workflow.build(), inputs={}
+        )
+        assert "update_meeting" in results
 
     @pytest.mark.asyncio
     async def test_update_datetime_with_microseconds(self, tmp_path):
@@ -233,7 +240,7 @@ class TestBugFix2_UpdateNodeDatetimeConversion:
             "LogUpdateNode",
             "update_log",
             {
-                "filter": {"message": "Test"},
+                "filter": {"id": "log-1"},  # UpdateNode requires id or record_id
                 "fields": {
                     "timestamp": "2024-09-15T12:30:45.123456"
                 },  # With microseconds
@@ -241,9 +248,10 @@ class TestBugFix2_UpdateNodeDatetimeConversion:
         )
 
         runtime = AsyncLocalRuntime()
-        results = await runtime.execute_workflow_async(workflow.build(), inputs={})
-        assert "results" in results
-        assert "update_log" in results["results"]
+        results, run_id = await runtime.execute_workflow_async(
+            workflow.build(), inputs={}
+        )
+        assert "update_log" in results
 
     @pytest.mark.asyncio
     async def test_update_datetime_with_offset(self, tmp_path):
@@ -262,7 +270,7 @@ class TestBugFix2_UpdateNodeDatetimeConversion:
             "TaskUpdateNode",
             "update_task",
             {
-                "filter": {"description": "Write docs"},
+                "filter": {"id": "task-1"},  # UpdateNode requires id or record_id
                 "fields": {
                     "due_date": "2024-10-01T18:00:00+05:30"
                 },  # With timezone offset
@@ -270,9 +278,10 @@ class TestBugFix2_UpdateNodeDatetimeConversion:
         )
 
         runtime = AsyncLocalRuntime()
-        results = await runtime.execute_workflow_async(workflow.build(), inputs={})
-        assert "results" in results
-        assert "update_task" in results["results"]
+        results, run_id = await runtime.execute_workflow_async(
+            workflow.build(), inputs={}
+        )
+        assert "update_task" in results
 
     @pytest.mark.asyncio
     async def test_update_datetime_keeps_datetime_objects(self, tmp_path):
@@ -299,7 +308,9 @@ class TestBugFix2_UpdateNodeDatetimeConversion:
             "AppointmentUpdateNode",
             "update_appointment",
             {
-                "filter": {"patient": "John Doe"},
+                "filter": {
+                    "id": "appointment-1"
+                },  # UpdateNode requires id or record_id
                 "fields": {
                     # Use ISO string instead of datetime object for JSON serialization
                     "appointment_time": "2024-12-25T09:00:00"  # ISO string, not datetime object
@@ -308,9 +319,10 @@ class TestBugFix2_UpdateNodeDatetimeConversion:
         )
 
         runtime = AsyncLocalRuntime()
-        results = await runtime.execute_workflow_async(workflow.build(), inputs={})
-        assert "results" in results
-        assert "update_appointment" in results["results"]
+        results, run_id = await runtime.execute_workflow_async(
+            workflow.build(), inputs={}
+        )
+        assert "update_appointment" in results
 
 
 class TestBugFix2_AllNodesDatetimeConversion:
@@ -445,11 +457,12 @@ class TestBugFix2_AllNodesDatetimeConversion:
         )
 
         runtime = AsyncLocalRuntime()
-        results = await runtime.execute_workflow_async(workflow.build(), inputs={})
+        results, run_id = await runtime.execute_workflow_async(
+            workflow.build(), inputs={}
+        )
 
         # Verify bulk creation succeeded with all datetime conversions
-        assert "results" in results
-        assert "bulk_create" in results["results"]
+        assert "bulk_create" in results
 
     @pytest.mark.asyncio
     async def test_bulk_update_filter_based_datetime_conversion(self, tmp_path):
@@ -475,11 +488,12 @@ class TestBugFix2_AllNodesDatetimeConversion:
         )
 
         runtime = AsyncLocalRuntime()
-        results = await runtime.execute_workflow_async(workflow.build(), inputs={})
+        results, run_id = await runtime.execute_workflow_async(
+            workflow.build(), inputs={}
+        )
 
         # Verify bulk update succeeded with datetime conversion
-        assert "results" in results
-        assert "bulk_update" in results["results"]
+        assert "bulk_update" in results
 
     @pytest.mark.asyncio
     async def test_bulk_update_data_based_datetime_conversion(self, tmp_path):
@@ -514,11 +528,12 @@ class TestBugFix2_AllNodesDatetimeConversion:
         )
 
         runtime = AsyncLocalRuntime()
-        results = await runtime.execute_workflow_async(workflow.build(), inputs={})
+        results, run_id = await runtime.execute_workflow_async(
+            workflow.build(), inputs={}
+        )
 
         # Verify data-based bulk update succeeded
-        assert "results" in results
-        assert "bulk_update" in results["results"]
+        assert "bulk_update" in results
 
     @pytest.mark.asyncio
     async def test_bulk_upsert_datetime_conversion(self, tmp_path):
@@ -553,11 +568,12 @@ class TestBugFix2_AllNodesDatetimeConversion:
         )
 
         runtime = AsyncLocalRuntime()
-        results = await runtime.execute_workflow_async(workflow.build(), inputs={})
+        results, run_id = await runtime.execute_workflow_async(
+            workflow.build(), inputs={}
+        )
 
         # Verify bulk upsert succeeded with datetime conversions
-        assert "results" in results
-        assert "bulk_upsert" in results["results"]
+        assert "bulk_upsert" in results
 
     @pytest.mark.asyncio
     async def test_all_nodes_backward_compatibility(self, tmp_path):
@@ -579,16 +595,16 @@ class TestBugFix2_AllNodesDatetimeConversion:
             "RecordUpdateNode",
             "update",
             {
-                "filter": {"title": "Record 1"},
+                "filter": {"id": "record-1"},  # UpdateNode requires id or record_id
                 "fields": {"timestamp": "2024-02-01T00:00:00"},
             },
         )
 
         runtime = AsyncLocalRuntime()
-        r2 = await runtime.execute_workflow_async(workflow2.build(), inputs={})
+        r2, run_id2 = await runtime.execute_workflow_async(workflow2.build(), inputs={})
 
         # UpdateNode should succeed - backward compatible
-        assert "results" in r2 and "update" in r2["results"]
+        assert "update" in r2
 
         # CreateNode test - allow JSON serialization errors (separate concern)
         workflow1 = WorkflowBuilder()
@@ -645,6 +661,411 @@ class TestBugFix2_AllNodesDatetimeConversion:
                 raise  # Real error, not just serialization
 
 
+class TestBugFix514_OptionalTypePreservation:
+    """Regression tests for BUG #514: Optional[T] Type Stripping
+
+    Tests that Optional[dict], Optional[list], and other Optional fields
+    are correctly handled with required=False in NodeParameter.
+
+    GitHub Issue: #514
+    """
+
+    @pytest.mark.asyncio
+    async def test_optional_dict_field_can_be_omitted(self, tmp_path):
+        """Test that Optional[dict] fields can be omitted during creation"""
+        from typing import Optional
+
+        db_path = tmp_path / "test_optional_dict.db"
+        db = DataFlow(database_url=f"sqlite:///{db_path}", reset_on_start=True)
+
+        @db.model
+        class Article:
+            id: str
+            title: str
+            metadata: Optional[dict] = None
+
+        # Create without metadata - should work!
+        workflow = WorkflowBuilder()
+        workflow.add_node(
+            "ArticleCreateNode",
+            "create",
+            {
+                "id": "article-1",
+                "title": "Hello World",
+                # No metadata - should be OK
+            },
+        )
+
+        runtime = AsyncLocalRuntime()
+        results, run_id = await runtime.execute_workflow_async(
+            workflow.build(), inputs={}
+        )
+
+        # Verify creation succeeded
+        assert "create" in results
+        # metadata should be None
+        assert results["create"].get("metadata") is None
+
+    @pytest.mark.asyncio
+    async def test_optional_dict_field_accepts_dict(self, tmp_path):
+        """Test that Optional[dict] fields accept dict values"""
+        from typing import Optional
+
+        db_path = tmp_path / "test_optional_dict2.db"
+        db = DataFlow(database_url=f"sqlite:///{db_path}", reset_on_start=True)
+
+        @db.model
+        class Article:
+            id: str
+            title: str
+            metadata: Optional[dict] = None
+
+        # Create with metadata dict
+        workflow = WorkflowBuilder()
+        workflow.add_node(
+            "ArticleCreateNode",
+            "create",
+            {
+                "id": "article-2",
+                "title": "With Metadata",
+                "metadata": {"author": "Alice", "tags": ["tech"]},
+            },
+        )
+
+        runtime = AsyncLocalRuntime()
+        results, run_id = await runtime.execute_workflow_async(
+            workflow.build(), inputs={}
+        )
+
+        # Verify creation succeeded with dict
+        assert "create" in results
+        # metadata should be dict (not string!)
+        result_metadata = results["create"].get("metadata")
+        assert isinstance(result_metadata, dict)
+        assert result_metadata["author"] == "Alice"
+
+    @pytest.mark.asyncio
+    async def test_optional_list_field_can_be_omitted(self, tmp_path):
+        """Test that Optional[list] fields can be omitted"""
+        from typing import Optional
+
+        db_path = tmp_path / "test_optional_list.db"
+        db = DataFlow(database_url=f"sqlite:///{db_path}", reset_on_start=True)
+
+        @db.model
+        class Product:
+            id: str
+            name: str
+            tags: Optional[list] = None
+
+        # Create without tags
+        workflow = WorkflowBuilder()
+        workflow.add_node(
+            "ProductCreateNode",
+            "create",
+            {
+                "id": "prod-1",
+                "name": "Widget",
+                # No tags
+            },
+        )
+
+        runtime = AsyncLocalRuntime()
+        results, run_id = await runtime.execute_workflow_async(
+            workflow.build(), inputs={}
+        )
+
+        assert "create" in results
+
+    @pytest.mark.asyncio
+    async def test_update_optional_field_to_none(self, tmp_path):
+        """Test that optional fields can be set to None via UpdateNode"""
+        from typing import Optional
+
+        db_path = tmp_path / "test_update_optional.db"
+        db = DataFlow(database_url=f"sqlite:///{db_path}", reset_on_start=True)
+
+        @db.model
+        class Document:
+            id: str
+            title: str
+            metadata: Optional[dict] = None
+
+        # Create with metadata, then clear it
+        workflow = WorkflowBuilder()
+        workflow.add_node(
+            "DocumentCreateNode",
+            "create",
+            {"id": "doc-1", "title": "Test Doc", "metadata": {"status": "draft"}},
+        )
+        workflow.add_node(
+            "DocumentUpdateNode",
+            "update",
+            {"filter": {"id": "doc-1"}, "fields": {"metadata": None}},  # Clear metadata
+        )
+        workflow.add_connection("create", "id", "update", "filter.id")
+
+        runtime = AsyncLocalRuntime()
+        results, run_id = await runtime.execute_workflow_async(
+            workflow.build(), inputs={}
+        )
+
+        # Verify update succeeded
+        assert "create" in results
+        assert "update" in results
+
+
+class TestBugFix515_DictListPreservation:
+    """Regression tests for BUG #515: Premature JSON Serialization
+
+    Tests that dict and list parameters remain as native Python objects
+    during validation, not prematurely serialized to JSON strings.
+
+    GitHub Issue: #515
+    """
+
+    @pytest.mark.asyncio
+    async def test_dict_parameter_not_serialized(self, tmp_path):
+        """Test that dict parameters remain as dicts during validation"""
+        db_path = tmp_path / "test_dict_not_serialized.db"
+        db = DataFlow(database_url=f"sqlite:///{db_path}", reset_on_start=True)
+
+        @db.model
+        class Config:
+            id: str
+            settings: dict
+
+        # Create with dict parameter
+        workflow = WorkflowBuilder()
+        workflow.add_node(
+            "ConfigCreateNode",
+            "create",
+            {"id": "cfg-1", "settings": {"theme": "dark", "lang": "en"}},
+        )
+
+        runtime = AsyncLocalRuntime()
+        results, run_id = await runtime.execute_workflow_async(
+            workflow.build(), inputs={}
+        )
+
+        # Verify creation succeeded
+        assert "create" in results
+        # settings should be dict, not string
+        result_settings = results["create"].get("settings")
+        assert isinstance(result_settings, dict)
+        assert result_settings["theme"] == "dark"
+
+    @pytest.mark.asyncio
+    async def test_list_parameter_not_serialized(self, tmp_path):
+        """Test that list parameters remain as lists during validation"""
+        db_path = tmp_path / "test_list_not_serialized.db"
+        db = DataFlow(database_url=f"sqlite:///{db_path}", reset_on_start=True)
+
+        @db.model
+        class Product:
+            id: str
+            name: str
+            tags: list
+
+        # Create with list parameter
+        workflow = WorkflowBuilder()
+        workflow.add_node(
+            "ProductCreateNode",
+            "create",
+            {
+                "id": "prod-1",
+                "name": "Widget",
+                "tags": ["electronics", "gadgets", "new"],
+            },
+        )
+
+        runtime = AsyncLocalRuntime()
+        results, run_id = await runtime.execute_workflow_async(
+            workflow.build(), inputs={}
+        )
+
+        # Verify creation succeeded
+        assert "create" in results
+        # tags should be list, not string
+        result_tags = results["create"].get("tags")
+        assert isinstance(result_tags, list)
+        assert len(result_tags) == 3
+
+    @pytest.mark.asyncio
+    async def test_empty_dict_not_serialized(self, tmp_path):
+        """Test that empty dicts {} remain as dicts, not strings"""
+        db_path = tmp_path / "test_empty_dict.db"
+        db = DataFlow(database_url=f"sqlite:///{db_path}", reset_on_start=True)
+
+        @db.model
+        class Session:
+            id: str
+            user_id: str
+            data: dict
+
+        # Create with empty dict
+        workflow = WorkflowBuilder()
+        workflow.add_node(
+            "SessionCreateNode",
+            "create",
+            {"id": "sess-1", "user_id": "user-1", "data": {}},  # Empty dict
+        )
+
+        runtime = AsyncLocalRuntime()
+        results, run_id = await runtime.execute_workflow_async(
+            workflow.build(), inputs={}
+        )
+
+        # Verify creation succeeded
+        assert "create" in results
+        # data should be dict, not string "{}"
+        result_data = results["create"].get("data")
+        assert isinstance(result_data, dict)
+        assert result_data == {}
+
+    @pytest.mark.asyncio
+    async def test_nested_dict_structure_preserved(self, tmp_path):
+        """Test that nested dict structures are preserved"""
+        db_path = tmp_path / "test_nested_dict.db"
+        db = DataFlow(database_url=f"sqlite:///{db_path}", reset_on_start=True)
+
+        @db.model
+        class UserProfile:
+            id: str
+            name: str
+            preferences: dict
+
+        # Create with nested dict
+        workflow = WorkflowBuilder()
+        workflow.add_node(
+            "UserProfileCreateNode",
+            "create",
+            {
+                "id": "profile-1",
+                "name": "Alice",
+                "preferences": {
+                    "notifications": {"email": True, "push": False},
+                    "theme": "dark",
+                },
+            },
+        )
+
+        runtime = AsyncLocalRuntime()
+        results, run_id = await runtime.execute_workflow_async(
+            workflow.build(), inputs={}
+        )
+
+        # Verify nested structure preserved
+        assert "create" in results
+        prefs = results["create"].get("preferences")
+        assert isinstance(prefs, dict)
+        assert prefs["notifications"]["email"] is True
+
+
+class TestBugFix514And515_Combined:
+    """Combined regression tests for BUG #514 and #515
+
+    Tests the interaction between Optional types and dict/list preservation,
+    which was the original failure scenario in Kaizen memory system.
+    """
+
+    @pytest.mark.asyncio
+    async def test_optional_dict_with_none(self, tmp_path):
+        """Test Optional[dict] can be None (the original bug scenario)"""
+        from typing import Optional
+
+        db_path = tmp_path / "test_combined.db"
+        db = DataFlow(database_url=f"sqlite:///{db_path}", reset_on_start=True)
+
+        @db.model
+        class Message:
+            id: str
+            content: str
+            metadata: Optional[dict] = None
+            created_at: datetime
+
+        # Test Case 1: Create without metadata (None)
+        workflow1 = WorkflowBuilder()
+        workflow1.add_node(
+            "MessageCreateNode",
+            "create1",
+            {
+                "id": "msg-1",
+                "content": "Test message",
+                "created_at": "2024-01-01T00:00:00Z",
+            },
+        )
+
+        runtime = AsyncLocalRuntime()
+        results1, run_id1 = await runtime.execute_workflow_async(
+            workflow1.build(), inputs={}
+        )
+
+        assert "create1" in results1
+
+        # Test Case 2: Create with metadata (dict)
+        workflow2 = WorkflowBuilder()
+        workflow2.add_node(
+            "MessageCreateNode",
+            "create2",
+            {
+                "id": "msg-2",
+                "content": "Test message 2",
+                "metadata": {"sender": "Alice", "priority": "high"},
+                "created_at": "2024-01-01T00:00:00Z",
+            },
+        )
+
+        results2, run_id2 = await runtime.execute_workflow_async(
+            workflow2.build(), inputs={}
+        )
+
+        assert "create2" in results2
+        # Metadata should be dict, not string
+        result_metadata = results2["create2"].get("metadata")
+        assert isinstance(result_metadata, dict)
+
+    @pytest.mark.asyncio
+    async def test_bulk_operations_with_optional_dict(self, tmp_path):
+        """Test bulk operations handle Optional[dict] correctly"""
+        from typing import Optional
+
+        db_path = tmp_path / "test_bulk_combined.db"
+        db = DataFlow(database_url=f"sqlite:///{db_path}", reset_on_start=True)
+
+        @db.model
+        class Event:
+            id: str
+            name: str
+            metadata: Optional[dict] = None
+
+        # Bulk create with mixed None and dict values
+        workflow = WorkflowBuilder()
+        workflow.add_node(
+            "EventBulkCreateNode",
+            "bulk_create",
+            {
+                "data": [
+                    {"id": "evt-1", "name": "Event 1"},  # No metadata
+                    {
+                        "id": "evt-2",
+                        "name": "Event 2",
+                        "metadata": {"type": "meeting"},
+                    },  # With metadata
+                    {"id": "evt-3", "name": "Event 3", "metadata": {}},  # Empty dict
+                ]
+            },
+        )
+
+        runtime = AsyncLocalRuntime()
+        results, run_id = await runtime.execute_workflow_async(
+            workflow.build(), inputs={}
+        )
+
+        # Verify bulk creation succeeded
+        assert "bulk_create" in results
+
+
 @pytest.fixture
 def cleanup_db_files():
     """Cleanup any database files created during tests"""
